/*
 *  Types/Note.cs
 *  Copyright (C) 2006-2007 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using System.Xml;
using System.Reflection;
using Incollector.GUI.Widgets;
using Gtk;
using Mono.Unix;

namespace Incollector.DataTypes {

	public class Word : DataType {
		
		protected static Gtk.Entry LanguageWidget = new Gtk.Entry ();
		protected static Gtk.Entry TranslationWidget = new Gtk.Entry ();
		protected static Gtk.Entry WordWidget = new Gtk.Entry ();
		
		protected string _DataLanguage = "";
		protected string _DataTranslation = "";
		protected string _DataWord = "";
		
		protected string DataLanguage {
			get { return this._DataLanguage;  }
			set { this._DataLanguage = value; }
		}
		
		protected string DataTranslation {
			get { return this._DataTranslation;  }
			set { this._DataTranslation = value; }
		}
		
		protected string DataWord {
			get { return this._DataWord;  }
			set { this._DataWord = value; }
		}
		
		
		
		public override string Name {
			get { return "word"; }
		}
		
		public override string Title {
			get { return Catalog.GetString ("Word"); }
		}
		
		public override string MenuText {
			get { return Catalog.GetString ("Add a word"); }
		}
		
		public override Gtk.Widget DefaultFocusWidget {
			get { return LanguageWidget; }
		}
		
		public override Gtk.Widget FocusWidget {
			get { return LanguageWidget; }
		}
		
		public override Gtk.Widget EditWidget {
			get {
				Gtk.Table tbl = new Gtk.Table (2, 3, false);
				
				Gtk.Label lbl1 = new Gtk.Label (Catalog.GetString ("Language"));
				lbl1.Xalign = 1;
				lbl1.Yalign = (float) 0.5;
				tbl.Attach (lbl1, 0, 1, 0, 1, AttachOptions.Fill,
							AttachOptions.Fill, 3, 3);
				
				LanguageWidget = new Gtk.Entry ();
				LanguageWidget.Text = this.DataLanguage;
				Incollector.Engine.Autocompletion.Assign (LanguageWidget, "data:" + Name + ":language");
				tbl.Attach (LanguageWidget, 1, 2, 0, 1,
							AttachOptions.Expand | AttachOptions.Fill,
							AttachOptions.Shrink | AttachOptions.Fill, 3, 3);
				
				
				Gtk.Label lbl2 = new Gtk.Label (Catalog.GetString ("Word"));
				lbl2.Xalign = 1;
				lbl2.Yalign = (float) 0.5;
				tbl.Attach (lbl2, 0, 1, 1, 2, AttachOptions.Fill,
							AttachOptions.Fill, 3, 3);
				
				WordWidget = new Gtk.Entry ();
				WordWidget.Text = this.DataWord;
				Incollector.Engine.Autocompletion.Assign (WordWidget, "data:" + Name + ":word");
				tbl.Attach (WordWidget, 1, 2, 1, 2,
							AttachOptions.Expand | AttachOptions.Fill,
							AttachOptions.Shrink | AttachOptions.Fill, 3, 3);
				
				
				Gtk.Label lbl3 = new Gtk.Label (Catalog.GetString ("Translation"));
				lbl2.Xalign = 1;
				lbl2.Yalign = (float) 0.5;
				tbl.Attach (lbl3, 0, 1, 2, 3, AttachOptions.Fill,
							AttachOptions.Fill, 3, 3);
				
				TranslationWidget = new Gtk.Entry ();
				TranslationWidget.Text = this.DataTranslation;
				Incollector.Engine.Autocompletion.Assign (TranslationWidget, "data:" + Name + ":translation");
				tbl.Attach (TranslationWidget, 1, 2, 2, 3,
							AttachOptions.Expand | AttachOptions.Fill,
							AttachOptions.Shrink | AttachOptions.Fill, 3, 3);
				
				tbl.ShowAll ();
				return tbl;
			}
		}
		
		public override string EditWidgetContent {
			set { this.DataWord = value; }
		}
		
		public override Gtk.Widget PreviewWidget {
			get {
				Gtk.Table tbl = new Gtk.Table (2, 2, false);
				
				Gtk.Label lbl1 = new Gtk.Label ();
				lbl1.UseMarkup = true;
				lbl1.Markup    = "<b>" + Catalog.GetString ("Language") + "</b>";
				lbl1.Xalign    = 1;
				tbl.Attach (lbl1, 0, 1, 0, 1, AttachOptions.Fill,
							AttachOptions.Fill, 3, 3);
				
				PreviewLabel Language  = new PreviewLabel (this.DataLanguage);
				Language.Xalign     = 0;
				Language.Yalign     = 0;
				tbl.Attach (Language, 1, 2, 0, 1, AttachOptions.Fill, AttachOptions.Fill, 3, 3);
				
				Gtk.Label lbl2 = new Gtk.Label ();
				lbl2.UseMarkup = true;
				lbl2.Markup    = "<b>" + Catalog.GetString ("Word") + "</b>";
				lbl2.Xalign    = 1;
				lbl2.Yalign    = 0;
				tbl.Attach (lbl2, 0, 1, 1, 2, AttachOptions.Fill,
							AttachOptions.Fill, 3, 3);
				
				PreviewLabel Word  = new PreviewLabel (this.DataWord);
				Word.Xalign     = 0;
				Word.Yalign     = 0;
				tbl.Attach (Word, 1, 2, 1, 2, AttachOptions.Fill, AttachOptions.Fill, 3, 3);
							
				Gtk.Label lbl3 = new Gtk.Label ();
				lbl3.UseMarkup = true;
				lbl3.Markup    = "<b>" + Catalog.GetString ("Translation") + "</b>";
				lbl3.Xalign    = 1;
				lbl3.Yalign    = 0;
				tbl.Attach (lbl3, 0, 1, 2, 3, AttachOptions.Fill,
							AttachOptions.Fill, 3, 3);
				
				PreviewLabel Translation  = new PreviewLabel (this.DataTranslation);
				Translation.Xalign     = 0;
				Translation.Yalign     = 0;
				tbl.Attach (Translation, 1, 2, 2, 3,
							AttachOptions.Expand | AttachOptions.Fill,
							AttachOptions.Expand | AttachOptions.Fill, 3, 3);
				
				tbl.ShowAll ();
				
				return tbl;
			}
		}
		
		public override string EntryTitle {
			get { return this.DataWord + " - " + this.DataTranslation; }
		}
		
		public override Gdk.Pixbuf Icon16 {
			get { return new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "word-16.png"); }
		}
		
		public override Gdk.Pixbuf Icon32 {
			get { return new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "word-32.png"); }
		}
		
		public override Gdk.Pixbuf Icon48 {
			get { return new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "word-48.png"); }
		}
		
		
		
		public Word ()
		{
		}
		
		
		public override DataType New ()
		{
			return new Word ();
		}
		
		
		public override DataType Clone ()
		{
			Word wrd = new Word ();
			
			wrd._DataLanguage = this._DataLanguage.Clone () as string;
			wrd._DataTranslation = this._DataTranslation.Clone () as string;
			wrd._DataWord = this._DataWord.Clone () as string;
			
			return wrd;
		}


		public override void ReadXml (XmlTextReader reader)
		{
			this.DataLanguage    = reader.GetAttribute ("language");
			this.DataWord        = reader.GetAttribute ("word");
			this.DataTranslation = reader.ReadString ();
			
			Incollector.Engine.Autocompletion.Add ("data:" + Name + ":language", this.DataLanguage);
			Incollector.Engine.Autocompletion.Add ("data:" + Name + ":word", this.DataWord);
			Incollector.Engine.Autocompletion.Add ("data:" + Name + ":translation", this.DataTranslation);
		}
		
		
		public override void WriteXml (XmlTextWriter writer)
		{
			writer.WriteStartElement (Name);
				writer.WriteAttributeString ("language", this.DataLanguage);
				writer.WriteAttributeString ("word", this.DataWord);
				writer.WriteString (this.DataTranslation);
			writer.WriteEndElement ();
		}
		
		
		public override void AddChangedEvent (EventHandler hdlr)
		{
			LanguageWidget.Changed    += hdlr;
			WordWidget.Changed        += hdlr;
			TranslationWidget.Changed += hdlr;
		}
		
		
		public override bool Search (string query)
		{
			return Engine.Text.Search (query, this.DataLanguage) || Engine.Text.Search (query, this.DataWord) || Engine.Text.Search (query, this.DataTranslation);
		}
		
		
		public override bool IsFormFilledIn ()
		{
			return (LanguageWidget.Text != "") && (WordWidget.Text != "") && (TranslationWidget.Text != "");
		}
		
		
		public override void SynchronizeData ()
		{
			this.DataLanguage = LanguageWidget.Text;
			this.DataTranslation = TranslationWidget.Text;
			this.DataWord = WordWidget.Text;
		}

	}
}
